__title__ = 'DRF Guard'
__description__ = 'Flexible permissions for Django REST Framework'
__url__ = 'https://github.com/yezyilomo/drf-guard'
__version__ = '0.1.1'
__author__ = 'Yezy Ilomo'
__author_email__ = 'yezileliilomo@hotmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2020 Yezy Ilomo'

# Version synonym
VERSION = __version__
